/**
 * Simple distance metric calculating the cosine of the angle between
 * two chunks' feature vectors.
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */

public class CosineDist extends ChunkDist 
{
    public CosineDist() {}
    public CosineDist(ChunkDist o) { super(o); }
    public CosineDist(int[] i) { super(i); }
    public CosineDist(ChunkDist o, int[] i) { super(o,i); }

    public double distance(Chunk ch1, Chunk ch2) 
    {
        FeatChunk c1 = null;
        FeatChunk c2 = null; 

        // this only works on FeatChunks
        try
        {
            c1 = (FeatChunk)ch1;
            c2 = (FeatChunk)ch2;
        }
        catch(ClassCastException e)
        {
            return super.distance(ch1, ch2);
        }

        double[] f1 = c1.getFeatures(featdim);
        double[] f2 = c2.getFeatures(featdim);
        double dist = 0;
        double normf1 = 0;
        double normf2 = 0;

        for(int i = 0; i < f1.length; i++)
        {
            dist += f1[i]*f2[i];
            normf1 +=  f1[i]*f1[i];
            normf2 +=  f2[i]*f2[i];
        }
        
        dist = super.distance(ch1, ch2) + dist/Math.sqrt(normf1*normf2);

        return dist;
    }
}
