import java.util.*;
/**
 * Representation of an EDLFile Chunk
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */

public class EDLChunk extends Chunk
{
    double dstTime;
    Vector commands; 

    private double[] features = null;
 
    /**
     * EDLFile Chunk constructor
     */
    public EDLChunk(String sf, double st, double l, double dt)
    {
        super(sf,st,l);
        dstTime = dt;

        commands = new Vector();
    }
   
    public EDLChunk(Chunk c, double dt)
    {
        this(c.srcFile, c.startTime, c.length, dt);
    }

    public EDLChunk(FeatChunk c, double dt)
    {
        this(c.srcFile, c.startTime, c.length, dt);
        setFeatures(c.getFeatures());
    }

    /**
     * The EDLChunk set/getFeatures methods are really only to
     * associate each EDL chunk with a set of features for
     * visualization purposes (i.e. with ImagePanel, to see how the
     * composers rearrange things) since the features associated with
     * chunks in EDLs are basically irrelevant.
     */
    public void setFeatures(double[] f)
    {
        features = f;
    }

    /**
     * Get the features associated with this chunk
     */
    public double[] getFeatures()
    {
        return features;
    }
    
    /**
     * Get the subset of features corresponding to the dimensions
     * listed in idx.
     */
    public double[] getFeatures(int[] idx)
    {
        if (idx == null)
            return getFeatures();

        double[] feats = new double[idx.length];

        for(int i = 0; i < idx.length; i++)
            feats[i] = features[idx[i]];

        return feats;
    }

    /**
     * Compare one EDLChunk to another - comparisons are
     * performed based on dstTime field
     */
    public int compareTo(Object o) throws ClassCastException
    {
        try
        {
            EDLChunk c = (EDLChunk)o;
            return Double.compare(dstTime, c.dstTime);
        }
        catch(ClassCastException e)
        {
            return super.compareTo(o);
        }
    }

    /**
     * Write a description of this Chunk as a String in the format
     * expected in an EDLFile
     */
    public String toString()
    {
        StringBuffer s = new StringBuffer(200);
        s.append(dstTime).append(" ").append(srcFile).append(" ");
        s.append(startTime).append(" ").append(length).append(" ");

        if(commands != null)
        {
            Iterator x = commands.iterator();
            while(x.hasNext())
                s.append(x.next()).append(" ");
        }

        s.append(comment).append("\n");
        
        return s.toString();
    }
}

