import java.io.*;

/**
 * Representation of a MEAPsoft file.
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */

public abstract class MEAPFile implements Serializable
{
    protected String filename;

    /**
     * Parse this MEAPFile.
     */
    public abstract void readFile() throws IOException, ParserException;

    /**
     * Write out properly formatted representation of this object
     */
    protected abstract void write(Writer w) throws IOException;

    /**
     * Write the file represented by this object.
     */
    public void writeFile() throws IOException
    {
        writeFile(filename);
    }

    /**
     * Write the file represented by this object.
     */
    public void writeFile(String fn) throws IOException
    {
        writeFile(fn, false);
    }

    /**
     * Write the file represented by this object.
     *
     * If append is false, fn will be overwritten.
     */
    public void writeFile(String fn, boolean append) throws IOException
    {
        if(fn == null)
            fn = filename;

        // only append if we are outputting to a different file
        BufferedWriter out = new BufferedWriter(new FileWriter(fn, append));
        write(out);
        out.close();
    }

    /**
     * Returns a String representing the properly formatted contents
     * of this file
     */
    public String toString()
    {
        StringWriter s = new StringWriter();

        try { write(s); }
        // this can't happen on a StringWriter, but we must appease Java
        catch(IOException e) {;}  

        return s.toString();
    }
}
