import java.util.*;
import javax.sound.sampled.*;
import java.io.FileWriter;
import java.io.IOException;

import featextractors.*;

/**
 * Helper class for Segmenter.java - does all of the work in fact.
 * Processes an audio stream, finds onsets, and writes them to a file.
 * Based on Mike Mandel's Extractor.java
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 * @author Mike Mandel (mim@ee.columbia.edu)
 */

public class SegmentExtractor implements FrameListener, Runnable, OnsetListener 
{
    Vector consumers = new Vector();
    STFT stft;
    OnsetDetector onsDetect;
    long lastOnset = -1;
    int minChunkLen = 20;
    //int maxChunkLen = 85
    // no max length between onsets
    int maxChunkLen = Integer.MAX_VALUE; 
    
    // keep track of whether or not we have seen any onsets yet...
    boolean recording = false;
    
    FeatFile outFile;
    String sourceFileName;
    
    public SegmentExtractor(STFT stft, OnsetDetector od, String sfn, FeatFile of)
    {
        this.stft = stft;
        onsDetect = od;
        sourceFileName = sfn;
        outFile = of;

        //maxChunkLen = stft.getRows();
    }
    
    public void run() 
    {
        stft.start();
    }
    
    /**
     * Callback function for OnsetDetector - processes a new onset
     */
    public void newOnset(long nextOnset, int zeroFrames) 
    {
        if(!recording) 
        {
            // Start recording at this onset, don't do anything else
            lastOnset = nextOnset;
            recording = true;
        } 
        else if(nextOnset - lastOnset < minChunkLen) 
        {
            // Ignore onsets that come too close together
        } 
        else 
        {
            // Got new onset, make a chunk from the last onset to this one
            outFile.chunks.add(new Chunk(sourceFileName, 
                                         stft.fr2Seconds(lastOnset), 
                                         stft.fr2Seconds(nextOnset-lastOnset)));
            lastOnset = nextOnset;
        }    
    }

    /**
     * Callback function for FrameListenerOnsetDetector - end
     * previous onset if we've reached the max chunk length
     */
    public void newFrame(STFT ignored, long frAddr) 
    {
        if(recording && frAddr - lastOnset > maxChunkLen) 
        {
            // Reached max length of chunk from last onset, make chunk
            recording = false;
            
            outFile.chunks.add(new Chunk(sourceFileName, 
                                         stft.fr2Seconds(lastOnset), 
                                         stft.fr2Seconds(maxChunkLen)));
        }
    }
}
