import javax.sound.sampled.*;
import java.io.*;
import java.util.*;
import java.util.regex.*;
import gnu.getopt.*;  

import featextractors.*;

/**
 * Program that processes a MEAPsoft EDL file and synthesizes and
 * audio file from it. 
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */

public class Synthesizer extends MEAPUtil
{
    EDLFile edlFile;
    String outFile = null;
    AudioWriter output;

    // output buffer
    double[] outSamples;


    // \todo{test output to line out}
    public void printUsageAndExit() 
    {
        System.out.println("Usage: Synthesizer [-options] file.edl \n\n" + 
                           "  where options include:\n" + 
                           "    -o output_file  output sound file (defaults to line out)\n" +
                           "");
        System.exit(0);
    }

    public Synthesizer(String infile, String outfile)
    {
        this(new EDLFile(infile), outfile); 
    }

    public Synthesizer(EDLFile edl, String outfile)
    {
        edlFile = edl;
        outFile = outfile; 
    }
    
    public Synthesizer(String[] args) 
    {
        if(args.length == 0)
            printUsageAndExit();

        // Parse arguments
        Getopt opt = new Getopt("Synthesizer", args, "o:");
        opt.setOpterr(false);
        
        int c = -1;
        while ((c = opt.getopt()) != -1) 
        {
            switch(c) 
            {
            case 'o':
                outFile = opt.getOptarg();
                break;
            case '?':
                printUsageAndExit();
                break;
            default:
                System.out.print("getopt() returned " + c + "\n");
            }
        }
        
        // parse arguments
        int ind = opt.getOptind();
        if(ind > args.length)
            printUsageAndExit();
        
        edlFile = new EDLFile(args[ind]);

        System.out.println("Synthesizing " + outFile + " from " + args[ind] + ".");
    }


    public void processEDL() throws IOException
    {
        // create output samples
        EDLChunk ch = (EDLChunk)edlFile.chunks.get(0); 
        outSamples = new double[(int)(samplingRate*(ch.dstTime + ch.length))];
        // outSamples should probably be initialized properly...
        Arrays.fill(outSamples, 0); 

        Iterator i = edlFile.chunks.iterator();
        while(i.hasNext())
        {
            EDLChunk currChunk = (EDLChunk)i.next();
            double[] chunkSamples = currChunk.getSamples(format);
            int offset = (int)(currChunk.dstTime*samplingRate);

            // \todo {should commands be parsed in Chunk.getSamples()?}
            Iterator c = currChunk.commands.iterator();
            while(c.hasNext())
            {
                String cmd = (String)c.next();
                if(cmd.equalsIgnoreCase("reverse"))
                {
                    for(int x = 0; x < chunkSamples.length/2; x++)
                    {
                        double tmp = chunkSamples[chunkSamples.length-x-1];
                        chunkSamples[chunkSamples.length-x-1] = chunkSamples[x];
                        chunkSamples[x] = tmp;
                    }
                }
                else
                    System.out.println("Ignored unknown command: " + cmd);
            }

            for(int x = 0; x < chunkSamples.length; x++)
            {
                // Smooth out any rough edges to the data with a simple triangular
                // window on each end
                // int winSize = ;
                //for(int i=0; i<winSize; i++) 
                //{
                //    chunkSamples[i] *= (double)i / (double)winSize;
                //    chunkSamples[samples.length-1-i] *= (double)i / (double)winSize;
                //}    
                
               outSamples[offset + x] += chunkSamples[x];
           }
        }    
    }
    
    public void go()
    {
        format = new AudioFormat(samplingRate, bitsPerSamp, numChannels, 
                                 signed, bigEndian);
        try 
        {
            edlFile.readFile();
            processEDL();
            
            // write out the audio data
            //AudioWriter aw = new AudioWriter(outFile, format,  AudioFileFormat.Type.WAVE);
            AudioWriter aw = openAudioWriter(outFile);
            
            aw.write(outSamples);
            aw.close();
        }
        catch(Exception e) 
        { 
            e.printStackTrace(); 
        }
    } 

    public static void main(String[] args) 
    {
        Synthesizer o2or = new Synthesizer(args);
        long startTime = System.currentTimeMillis();
        o2or.go();
        System.out.println("Done.  Took " + 
                           ((System.currentTimeMillis() - startTime)/1000.0)
                           + "s");
        System.exit(0);
    }
}
