package featextractors;

/**
 * A simple feature calculation that averages all spectral frames into
 * a single vector.
 *
 * @author Mike Mandel (mim@ee.columbia.edu)
 */

public class AvgSpec implements FeatureExtractor 
{
    public double[] features(STFT stft, long startFrame, int length) 
    {

        double[] avgSpec= new double[stft.getRows()];
        double[] curFrame;
        double sum = 0;
        
        for(int band=0; band<avgSpec.length; band++)
            avgSpec[band] = 0;
        
        for(int frame=0; frame<length; frame++) 
        {
            curFrame = stft.getFrame(startFrame+frame);
            for(int band=0; band<avgSpec.length; band++) 
            {
                double tmp = curFrame[band] / length;
                avgSpec[band] += tmp;
                sum += tmp;
            }
        }
        
        for(int band=0; band<avgSpec.length; band++)
            avgSpec[band] = avgSpec[band] / sum;
        
        return avgSpec;
    }
}
